/*
 * cimrpm.c
 *
 * (C) Copyright IBM Corp. 2002
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author: Viktor Mihajlovski <mihajlov@de.ibm.com>
 * Contributors:
 *
 * Description: RPM compatibility interface layer
 *            
 * The function of this layer is to dynamically load the right shared lib
 * for the RPM version installed on the current system.
 */

#include "cimrpm.h"
#include "cimrpmfp.h"
#include <stdio.h>
#include <dlfcn.h>

/* Function pointers to the cimrpm library */
struct _FPCRPM * fpCRPM; 

/* shared lib initialization routine */
void _init()
{
  void * dlhandle;
  fpCRPM=NULL;
  /* 
   * insert some smart version detection code
   * if (rpmversion < 4)
   */
  dlhandle = dlopen ( "libcimrpmv3.so", RTLD_LAZY );
  if (dlhandle) {
    fpCRPM = dlsym ( dlhandle, "FP_CRPM" );
    if (!fpCRPM) {
      fprintf(stderr,"Could not resolve symbol FP_CRPM, %s\n", dlerror());
    }
    return;
  } else { 
    dlhandle = dlopen ( "libcimrpmv4.so", RTLD_LAZY );
    if (dlhandle) {
      fpCRPM = dlsym ( dlhandle, "FP_CRPM" );
      if (!fpCRPM) {
	fprintf(stderr,"Could not resolve symbol FP_CRPM, %s\n", dlerror());
      }
      return;
    }
  }
  fprintf(stderr,"Could not load RPM support library, %s\n",dlerror());
}

/* shared lib deinitialization routine */
void _fini()
{
}

/* Open RPM database and return handle to it */
HCRPM CRPM_Init()
{
  /* Checking only for the init call and assuming that afterwards the fps
   * are set.
   */
  if (fpCRPM) 
    return fpCRPM->fpCRPM_Init();
  else
    return NULL;
}

/* Close RPM database, free handle memory */
void CRPM_Term(HCRPM dbptr)
{
  fpCRPM->fpCRPM_Term(dbptr);
}

/* Set up a new package enumeration */
int CRPM_InitPkgEnum(HCRPM dbptr,CRPM_SE* se)
{
  return fpCRPM->fpCRPM_InitPkgEnum(dbptr,se);
}

/* Get the next package in the enumeration */
int CRPM_NextPkgEnum(CRPM_SE* se)
{
  return fpCRPM->fpCRPM_NextPkgEnum(se);
}

/* Terminate package enumeration and free allocated memory */
void CRPM_TermPkgEnum(CRPM_SE* se)
{
  fpCRPM->fpCRPM_TermPkgEnum(se);
}

/* Retrieve single software element */
int CRPM_SinglePkgEnum(HCRPM hcrpm,const char * spec,CRPM_SE* se)
{
  return fpCRPM->fpCRPM_SinglePkgEnum(hcrpm,spec,se);
}

/* Retrieve list of files for software element */
char ** CRPM_SinglePkgFiles(HCRPM hcrpm, const char* spec, CRPM_SE* se)
{
  return fpCRPM->fpCRPM_SinglePkgFiles(hcrpm,spec,se);
}
