#ifndef _CIMRPM_H_
#define _CIMRPM_H_
/*
 * cimrpm.h
 *
 * (C) Copyright IBM Corp. 2002
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author: Viktor Mihajlovski <mihajlov@de.ibm.com>
 * Contributors:
 *
 * Description: RPM Database Access for CIM
 */

/* Software Element data obtained through RPM API */
typedef struct {
  char * seId;            /* RPMTAG_NAME */
  char * seName;          /* RPMTAG_NAME */
  char * seVersion;       /* RPMTAG_VERSION */
  int    seState;         /* 2 (installable) if from database */
  int    seOs;            /* RPMTAG_OS mapped to CIM integer */
  char * seManufacturer;  /* RPMTAG_VENDOR/PACKAGER? */
  char * seInstalled;     /* RPMTAG_INSTALLTIME */
  char * seCaption;       /* RPMTAG_SUMMARY */
  char * seDescription;   /* RPMTAG_DESCRIPTION */
  void * reserved;
} CRPM_SE;

/* RPM Handle */
typedef void * HCRPM;

/* Initialize CIM RPM session, returns a handle, NULL if failed */
HCRPM CRPM_Init();

/* Terminate CIM RPM session */
void CRPM_Term(HCRPM);

/* Initialize package enumeration, returns < 0 if error */
int CRPM_InitPkgEnum(HCRPM,CRPM_SE*);

/* Get next package, returns 0 if no more package, < 0 if error */
int CRPM_NextPkgEnum(CRPM_SE*);

/* Terminate package enumeration, release allocated memory */
void CRPM_TermPkgEnum(CRPM_SE*);

/* Retrieve single software element */
int CRPM_SinglePkgEnum(HCRPM,const char *,CRPM_SE*);

/* Retrieve list of files for software element */
char ** CRPM_SinglePkgFiles(HCRPM,const char *, CRPM_SE*);

#endif

