/*
 * cmpiOSBase_RpmAssociatedFile.c
 *
 * (C) Copyright IBM Corp. 2002, 2005
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author:       Viktor Mihajlovski <mihajlov@de.ibm.com>
 * Contributors: Heidi Neumann <heidineu@de.ibm.com>
 *
 * Interface Type : Common Manageability Programming Interface ( CMPI )
 *
 * Description: 
 * This is the factory implementation for creating instances of CIM
 * association Linux_RpmAssociatedFile.
 * 
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cmpidt.h"
#include "cmpimacs.h"

#include "cmpiOSBase_Common.h"
#include "OSBase_Common.h"

#include "cmpiOSBase_RpmFileCheck.h"
#include "cmpiOSBase_RpmAssociatedFile.h"


/* ---------------------------------------------------------------------------*/
/* private declarations                                                       */

/* ---------------------------------------------------------------------------*/


/* ---------------------------------------------------------------------------*/
/*                            Factory functions                               */
/* ---------------------------------------------------------------------------*/

/* ---------- method to create a CMPIObjectPath of Linux_RpmAssociatedFile ---*/

CMPIObjectPath * _makePath_RpmAssociatedFile( CMPIBroker * _broker, 
					   CMPIObjectPath * filecheck,
					   CMPIObjectPath * package,
					   CMPIStatus * rc) {
  CMPIObjectPath * op = NULL;
  CMPIInstance   * ci = NULL;

  _OSBASE_TRACE(4,("--- _makePath_RpmAssociatedFile() called"));

  ci = _makeInst_RpmAssociatedFile(_broker, filecheck, package, rc);
  if( ci == NULL ) { return NULL; }

  op = CMGetObjectPath(ci,rc); 
  CMSetNameSpace(op,CMGetCharPtr(CMGetNameSpace(package,rc)));

  _OSBASE_TRACE(4,("--- _makePath_RpmAssociatedFile() exited"));
  return op; 
}

/* ----------- method to create a CMPIInstance of Linux_RpmAssociatedFile ----*/

CMPIInstance * _makeInst_RpmAssociatedFile( CMPIBroker * _broker,
					 CMPIObjectPath * filecheck,
					 CMPIObjectPath * package,
					 CMPIStatus * rc) {
  CMPIObjectPath * op = NULL;
  CMPIInstance   * ci = NULL;

  _OSBASE_TRACE(4,("--- _makeInst_RpmAssociatedFile() called"));

  /* create CMPIObjectPath of this association <_AssocName> */
  op = CMNewObjectPath( _broker, CMGetCharPtr(CMGetNameSpace(package,rc)),
			_AssocName, rc );
  if( CMIsNullObject(op) ) { 
    CMSetStatusWithChars( _broker, rc, 
			  CMPI_RC_ERR_FAILED, "Create CMPIObjectPath failed." ); 
    goto exit;
  }

  ci = CMNewInstance( _broker, op, rc);
  if( CMIsNullObject(ci) ) { 
    CMSetStatusWithChars( _broker, rc, 
			  CMPI_RC_ERR_FAILED, "Create CMPIInstance failed." ); 
    _OSBASE_TRACE(4,("--- _makeInst_RpmAssociatedFile() failed : %s",CMGetCharPtr(rc->msg)));
    goto exit;
  }

  CMSetProperty( ci, _RefLeft, (CMPIValue*)&(filecheck), CMPI_ref ); 
  CMSetProperty( ci, _RefRight, (CMPIValue*)&(package), CMPI_ref );

 exit:
  _OSBASE_TRACE(4,("--- _makeInst_RpmAssociatedFile() exited"));
  return ci; 
}

/* ----- method to handle the association logic for Linux_RpmAssociatedFile --*/

int assoc_RpmAssociatedFile( CMPIBroker * _broker,
			  CMPIContext * ctx,
			  CMPIResult * rslt,
			  CMPIObjectPath * ref,
			  int assocType,
			  CMPIStatus * rc )
{
  CMPIString     * sourceClass = NULL;
  CMPIData         spec;
  CMPIObjectPath * op          = NULL;
  CMPIObjectPath * rop         = NULL;
  CMPIInstance   * rci         = NULL;
  CMPIInstance   * sci         = NULL;
  HCRPM            hcrpm;
  CRPM_SE          crpmse;
  char          ** filelist;
  int              i           = 0;

  _OSBASE_TRACE(2,("--- assoc_RpmAssociatedFile() called"));

  /* check if source instance does exist */
  sci = CBGetInstance(_broker, ctx, ref, NULL, rc);
  if( sci == NULL ) {
    if( rc->rc == CMPI_RC_ERR_FAILED ) {
      CMSetStatusWithChars( _broker, rc,
			    CMPI_RC_ERR_FAILED, "GetInstance of source object failed.");
    }
    if( rc->rc == CMPI_RC_ERR_NOT_FOUND ) {
      CMSetStatusWithChars( _broker, rc,
			    CMPI_RC_ERR_NOT_FOUND, "Source object not found.");
    }
    _OSBASE_TRACE(2,("--- assoc_RpmAssociatedFile() failed : %s",CMGetCharPtr(rc->msg)));
    return -1;
  }

  sourceClass = CMGetClassName(ref, rc);
  if (CMGetCharPtr(sourceClass) == NULL) {
    CMSetStatusWithChars( _broker, rc,
			  CMPI_RC_ERR_FAILED, "Could not get source class name.");
    _OSBASE_TRACE(2,("--- assoc_RpmAssociatedFile() failed : %s",CMGetCharPtr(rc->msg)));
    return -1;
  }

  /* source class is Linux_RpmFileCheck; target class is Linux_RpmPackage */
  if( strcasecmp(CMGetCharPtr(sourceClass),_RefLeftClass) == 0 ) {
    /* NOT_SUPPORTED - association is uni-directional */
  }
  /* source class is Linux_RpmPackage; target class is Linux_RpmFileCheck */
  else if( strcasecmp(CMGetCharPtr(sourceClass),_RefRightClass) == 0 ) {

    spec = CMGetKey(ref, "Name", rc);
    if( spec.value.string == NULL ) { 
      CMSetStatusWithChars( _broker, rc, 
			    CMPI_RC_ERR_FAILED, "Could not get Name of RPM Package" ); 
      _OSBASE_TRACE(2,("--- assoc_RpmAssociatedFile() failed : %s",CMGetCharPtr(rc->msg)));
      return -1;
    }

    hcrpm=CRPM_Init();
    if (!hcrpm) {
      _OSBASE_TRACE(2,("--- assoc_RpmAssociatedFile() exited : Could not find associated RPM file check."));
      return 0;
    }

    if (!CRPM_SinglePkgEnum(hcrpm,CMGetCharPtr(spec.value.string),&crpmse)) {
      filelist=CRPM_SinglePkgFiles(hcrpm,CMGetCharPtr(spec.value.string),&crpmse);
      if (filelist) {
	for (i=0; filelist[i]; i++) {
	  op = _makePath_RpmFileCheck( _broker, ctx, ref, &crpmse, 
				       filelist[i], rc );
	  if (assocType&ATYPE_ASSOC) {
	    /* the referenced object of Linux_RpmFileCheck */
	    if (assocType&ATYPE_NAME) {
	      CMReturnObjectPath(rslt, op); 
	    }
	    else {	  
	      rci = CBGetInstance(_broker, ctx, op, NULL, rc);
	      if(rci==NULL) { return -1; }
	      CMReturnInstance( rslt, rci );
	    }
	  } 
	  else {
	    /* the object of the reference Linux_RpmAssociatedFile */
	    if (assocType&ATYPE_NAME) {
	      rop = _makePath_RpmAssociatedFile(_broker, op, ref, rc);
	      CMReturnObjectPath(rslt, rop); 
	    } 
	    else {
	      rci = _makeInst_RpmAssociatedFile(_broker, op, ref, rc);
	      CMReturnInstance( rslt, rci );
	    }
	  }
	}
	free(filelist);
      }
      CRPM_TermPkgEnum(&crpmse);
    }
    CRPM_Term(hcrpm);
  }

  _OSBASE_TRACE(2,("--- assoc_RpmAssociatedFile() exited"));
  return 0;
}


/* ---------------------------------------------------------------------------*/
/*                     end of cmpiOSBase_RpmAssociatedFile.c                  */
/* ---------------------------------------------------------------------------*/

