/*
 * cmpiOSBase_RpmFileCheck.c
 *
 * (C) Copyright IBM Corp. 2002, 2005
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author:       Viktor Mihajlovski <mihajlov@de.ibm.com>
 * Contributors: Heidi Neumann <heidineu@de.ibm.com>
 *
 * Interface Type : Common Manageability Programming Interface ( CMPI )
 *
 * Description: 
 * This is the factory implementation for creating instances of CIM
 * class Linux_RpmFileCheck.
 * 
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cmpidt.h"
#include "cmpimacs.h"

#include "OSBase_Common.h"
#include "cmpiOSBase_Common.h"
#include "cmpiOSBase_RpmFileCheck.h"


/* ---------------------------------------------------------------------------*/
/* private declarations                                                       */

/* ---------------------------------------------------------------------------*/


/* ---------------------------------------------------------------------------*/
/*                            Factory functions                               */
/* ---------------------------------------------------------------------------*/

/* ---------- method to create a CMPIObjectPath of Linux_RpmFileCheck --------*/

CMPIObjectPath * _makePath_RpmFileCheck( CMPIBroker * _broker,
		 CMPIContext * ctx, 
	         CMPIObjectPath * ref,
	         CRPM_SE * crpmse,
		 char * filename,
	         CMPIStatus * rc) {
  CMPIObjectPath * op = NULL;
 
  _OSBASE_TRACE(2,("--- _makePath_RpmFileCheck() called"));

  op = CMNewObjectPath( _broker, CMGetCharPtr(CMGetNameSpace(ref,rc)), 
			_ClassName, rc );
  if( CMIsNullObject(op) ) { 
    CMSetStatusWithChars( _broker, rc, 
			  CMPI_RC_ERR_FAILED, "Create CMPIObjectPath failed." ); 
    _OSBASE_TRACE(2,("--- _makePath_RpmFileCheck() failed : %s",CMGetCharPtr(rc->msg)));
    goto exit; 
  }

  CMAddKey(op, "SoftwareElementID", crpmse->seId, CMPI_chars);
  CMAddKey(op, "Name", crpmse->seName, CMPI_chars);
  CMAddKey(op, "Version", crpmse->seVersion, CMPI_chars);
  CMAddKey(op, "SoftwareElementState", (CMPIValue*)&(crpmse->seState), CMPI_uint16);
  CMAddKey(op, "TargetOperatingSystem", (CMPIValue*)&(crpmse->seOs), CMPI_uint16);
  CMAddKey(op, "CheckID", filename, CMPI_chars);

 exit:
  _OSBASE_TRACE(2,("--- _makePath_RpmFileCheck() exited"));
  return op;             
}

/* ----------- method to create a CMPIInstance of Linux_RpmFileCheck ---------*/

CMPIInstance * _makeInst_RpmFileCheck( CMPIBroker * _broker,
	       CMPIContext * ctx, 
	       CMPIObjectPath * ref,
	       const char ** properties,
	       CRPM_SE * crpmse,
	       char * filename,
	       CMPIStatus * rc) {
  CMPIObjectPath *  op        = NULL;
  CMPIInstance   *  ci        = NULL;
  const char     ** keys      = NULL;
  int               keyCount  = 0;

  _OSBASE_TRACE(2,("--- _makeInst_RpmFileCheck() called"));

  op = CMNewObjectPath( _broker, CMGetCharPtr(CMGetNameSpace(ref,rc)), 
			_ClassName, rc );
  if( CMIsNullObject(op) ) { 
    CMSetStatusWithChars( _broker, rc, 
			  CMPI_RC_ERR_FAILED, "Create CMPIObjectPath failed." ); 
    _OSBASE_TRACE(2,("--- _makeInst_RpmFileCheck() failed : %s",CMGetCharPtr(rc->msg)));
    goto exit; 
  }

  ci = CMNewInstance( _broker, op, rc);
  if( CMIsNullObject(ci) ) { 
    CMSetStatusWithChars( _broker, rc, 
			  CMPI_RC_ERR_FAILED, "Create CMPIInstance failed." ); 
    _OSBASE_TRACE(2,("--- _makeInst_RpmFileCheck() failed : %s",CMGetCharPtr(rc->msg)));
    goto exit; 
  }

  /* set property filter */
  keys = calloc(7,sizeof(char*));
  keys[0] = strdup("SoftwareElementID");
  keys[1] = strdup("Name");
  keys[2] = strdup("Version");
  keys[3] = strdup("SoftwareElementState");
  keys[4] = strdup("TargetOperatingSystem");
  keys[5] = strdup("CheckID");
  CMSetPropertyFilter(ci,properties,keys);
  for( ;keys[keyCount]!=NULL;keyCount++) { free((char*)keys[keyCount]); }
  free(keys);

  CMSetProperty( ci, "SoftwareElementID", crpmse->seId, CMPI_chars );
  CMSetProperty( ci, "Name", crpmse->seName, CMPI_chars );
  CMSetProperty( ci, "Version", crpmse->seVersion, CMPI_chars );
  CMSetProperty( ci, "SoftwareElementState", (CMPIValue*)&(crpmse->seState), CMPI_uint16 );
  CMSetProperty( ci, "TargetOperatingSystem", (CMPIValue*)&(crpmse->seOs), CMPI_uint16 );
  CMSetProperty( ci, "CheckID", filename, CMPI_chars);

  CMSetProperty( ci, "FileName", filename, CMPI_chars);

 exit:
  _OSBASE_TRACE(2,("--- _makeInst_RpmFileCheck() exited"));
  return ci;
}


/* ---------------------------------------------------------------------------*/
/*                     end of cmpiOSBase_RpmFileCheck.c                       */
/* ---------------------------------------------------------------------------*/
